
# q-expansion of newform 224.3.d.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 224
weight = 3
poly_data = [4, 0, 56, 0, 53, 0, 14, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 2, 0, 0, 0, 0, 0, 0], 1], [[8, 0, 24, 0, 11, 0, 1, 0], 2], [[0, 42, 0, 51, 0, 14, 0, 1], 4], [[12, 0, 38, 0, 13, 0, 1, 0], 2], [[0, -32, 0, -42, 0, -13, 0, -1], 2], [[16, 0, 11, 0, 1, 0, 0, 0], 1], [[0, -110, 0, -77, 0, -16, 0, -1], 2]]

hecke_ring_character_values = [[127, [-1, 0, 0, 0, 0, 0, 0, 0]], [197, [1, 0, 0, 0, 0, 0, 0, 0]], [129, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, -2, 0, -1, 0, 1], [4, 0, 0, 0, 1, 0, 0, 0], [-2, 0, 0, 0, 0, 0, 2, 0], [0, 3, 0, 0, 0, 2, 0, 0], [0, -6, 0, -2, 0, 2, 0, 1], [-10, 0, 0, 0, 2, 0, 0, 0], [0, 6, 0, -4, 0, 0, 0, -2], [-22, 0, 6, 0, 0, 0, -2, 0], [18, 0, 6, 0, 0, 0, 2, 0], [0, 0, 0, 10, 0, 7, 0, 3], [0, 6, 0, 0, 0, -4, 0, 4], [6, 0, 6, 0, -2, 0, -2, 0], [0, 3, 0, -4, 0, -8, 0, 2], [-24, 0, 0, 0, 7, 0, -4, 0], [0, 6, 0, 18, 0, -13, 0, -1], [0, 0, 0, 4, 0, 4, 0, -6], [34, 0, 0, 0, 2, 0, -6, 0], [0, -12, 0, 8, 0, 18, 0, 0], [0, 3, 0, 28, 0, -14, 0, 2], [-10, 0, -6, 0, 0, 0, -4, 0], [66, 0, 6, 0, -6, 0, -4, 0], [-16, 0, 0, 0, -5, 0, 12, 0], [0, -6, 0, 28, 0, -4, 0, -6], [0, 18, 0, -22, 0, 1, 0, 3], [26, 0, 18, 0, 16, 0, 2, 0], [-20, 0, -3, 0, 7, 0, -7, 0], [0, 6, 0, -2, 0, 4, 0, 9], [0, -9, 0, -12, 0, 34, 0, 6], [58, 0, 0, 0, -12, 0, 0, 0], [0, 33, 0, 0, 0, -18, 0, 0], [-30, 0, -12, 0, -16, 0, 12, 0], [0, -30, 0, 10, 0, 2, 0, 3], [84, 0, 0, 0, -11, 0, -4, 0], [0, 0, 0, 10, 0, -7, 0, -5], [0, 42, 0, 8, 0, -36, 0, -4], [-76, 0, 12, 0, -15, 0, 12, 0], [0, -6, 0, 34, 0, -25, 0, -1], [36, 0, -12, 0, 29, 0, 8, 0], [0, -24, 0, -68, 0, 42, 0, -2], [32, 0, 15, 0, 1, 0, -9, 0], [6, 0, -6, 0, 30, 0, 10, 0], [0, -42, 0, 20, 0, 0, 0, -2], [0, 6, 0, 42, 0, 39, 0, -13], [0, 24, 0, 60, 0, -12, 0, -14], [0, 27, 0, -48, 0, -38, 0, -16], [-4, 0, -12, 0, 1, 0, -24, 0], [-94, 0, 0, 0, 4, 0, -12, 0], [0, -18, 0, -30, 0, -28, 0, -9], [18, 0, -18, 0, -2, 0, 0, 0], [0, 21, 0, -36, 0, 2, 0, 10], [50, 0, 24, 0, -20, 0, -4, 0], [0, -48, 0, 36, 0, 32, 0, 2], [-44, 0, 12, 0, -7, 0, 4, 0], [0, 36, 0, 20, 0, -24, 0, -18], [18, 0, -24, 0, -8, 0, -24, 0], [66, 0, -24, 0, 36, 0, -4, 0], [0, 15, 0, 88, 0, -16, 0, 12], [-176, 0, 12, 0, -1, 0, -12, 0], [0, -57, 0, 12, 0, 42, 0, -14], [0, 24, 0, 24, 0, 68, 0, 0], [-230, 0, 18, 0, -4, 0, 14, 0], [-146, 0, 30, 0, 2, 0, 14, 0], [0, 0, 0, -54, 0, -53, 0, -37], [80, 0, -33, 0, -43, 0, 27, 0], [0, -24, 0, -38, 0, 27, 0, -21], [-4, 0, 0, 0, 5, 0, -16, 0], [62, 0, -54, 0, -38, 0, -22, 0], [0, 54, 0, 22, 0, 42, 0, -11], [0, 120, 0, -60, 0, -16, 0, -2], [370, 0, 12, 0, 32, 0, -20, 0], [0, -36, 0, -54, 0, -5, 0, 27], [0, 102, 0, 96, 0, -76, 0, 12], [-262, 0, 6, 0, -8, 0, 30, 0], [-300, 0, 12, 0, -7, 0, 28, 0], [244, 0, 21, 0, -21, 0, -23, 0], [134, 0, -24, 0, -26, 0, 20, 0], [0, 21, 0, -48, 0, 96, 0, 0], [202, 0, 36, 0, -44, 0, 28, 0], [0, -30, 0, -122, 0, 8, 0, 13], [-562, 0, -12, 0, -40, 0, -2, 0], [0, -60, 0, -76, 0, -8, 0, 10], [0, 66, 0, 30, 0, -67, 0, 1], [290, 0, 0, 0, 32, 0, -8, 0], [40, 0, 27, 0, 41, 0, 31, 0], [100, 0, -24, 0, -23, 0, 40, 0], [0, 60, 0, 188, 0, 38, 0, -26], [0, -33, 0, 0, 0, -38, 0, -24], [0, 6, 0, 12, 0, 68, 0, -18], [0, -102, 0, -98, 0, 18, 0, 9], [0, 66, 0, 150, 0, -35, 0, 13], [0, -6, 0, -34, 0, -93, 0, -31], [0, 48, 0, 60, 0, 48, 0, -18], [388, 0, -36, 0, -15, 0, -36, 0], [-346, 0, 36, 0, 18, 0, -12, 0], [0, -57, 0, 236, 0, 64, 0, 26], [42, 0, 0, 0, -76, 0, 56, 0], [0, 84, 0, -190, 0, 13, 0, -17], [-26, 0, 42, 0, -66, 0, -30, 0], [0, 45, 0, 212, 0, -8, 0, -26], [-196, 0, -15, 0, -9, 0, -11, 0], [0, -132, 0, -62, 0, -49, 0, 23], [70, 0, 6, 0, 2, 0, 18, 0], [0, -27, 0, 0, 0, 42, 0, 0], [146, 0, 60, 0, 40, 0, -12, 0], [0, -48, 0, 160, 0, 24, 0, 36], [158, 0, 6, 0, 20, 0, 0, 0], [0, -24, 0, 8, 0, 76, 0, 20], [-114, 0, 84, 0, 14, 0, -12, 0], [56, 0, -93, 0, 17, 0, -17, 0], [0, -105, 0, 72, 0, 20, 0, -12], [0, 132, 0, -200, 0, 28, 0, 24], [-252, 0, -27, 0, -101, 0, -15, 0], [0, -27, 0, -252, 0, 24, 0, 6], [0, 6, 0, 68, 0, 52, 0, 46], [-290, 0, -24, 0, 62, 0, -32, 0], [0, -60, 0, 90, 0, -3, 0, -5], [128, 0, -36, 0, -21, 0, 24, 0], [82, 0, -12, 0, 100, 0, 32, 0], [244, 0, 36, 0, 81, 0, 4, 0], [0, -138, 0, -86, 0, 77, 0, -29], [0, -33, 0, -4, 0, 54, 0, 50], [134, 0, 12, 0, -78, 0, -4, 0], [338, 0, -54, 0, 84, 0, 42, 0], [0, -54, 0, -96, 0, 52, 0, -4], [0, -138, 0, -120, 0, 120, 0, 4], [-464, 0, -60, 0, -37, 0, -40, 0], [0, 0, 0, -246, 0, 25, 0, -37], [0, -186, 0, 270, 0, 54, 0, 33], [0, -6, 0, -18, 0, -52, 0, 1], [-426, 0, -24, 0, -78, 0, 32, 0], [122, 0, -18, 0, 124, 0, 2, 0], [-686, 0, 18, 0, 46, 0, 76, 0], [-520, 0, -48, 0, -17, 0, -12, 0], [0, 15, 0, -72, 0, -58, 0, 36], [580, 0, 36, 0, 97, 0, -20, 0], [168, 0, -45, 0, 25, 0, 55, 0], [0, 123, 0, 200, 0, -64, 0, 20], [-70, 0, 60, 0, 52, 0, 28, 0], [0, 36, 0, -164, 0, -216, 0, 6], [0, 162, 0, -6, 0, 105, 0, 19], [752, 0, 12, 0, 55, 0, -20, 0], [0, -6, 0, -188, 0, -140, 0, -58], [-300, 0, -48, 0, -51, 0, -52, 0], [598, 0, -120, 0, -22, 0, 48, 0], [0, 165, 0, -20, 0, 54, 0, -22], [0, 24, 0, -28, 0, 114, 0, -38], [-86, 0, 54, 0, 0, 0, -74, 0], [6, 0, 66, 0, 2, 0, -42, 0], [0, -138, 0, -2, 0, -53, 0, -7], [0, -210, 0, 120, 0, 92, 0, -12], [0, 102, 0, -2, 0, -99, 0, 17], [0, -30, 0, -222, 0, -84, 0, 15], [0, -156, 0, 332, 0, -44, 0, 22], [254, 0, -96, 0, -108, 0, 70, 0], [370, 0, 144, 0, 14, 0, 46, 0], [-320, 0, -36, 0, -49, 0, -28, 0], [0, -36, 0, 266, 0, 141, 0, -29], [-326, 0, -12, 0, -68, 0, -28, 0], [0, -198, 0, -202, 0, 206, 0, -43], [0, 171, 0, -92, 0, -208, 0, -26], [2, 0, -60, 0, -68, 0, -48, 0], [0, 138, 0, 148, 0, 44, 0, 38], [0, 24, 0, 152, 0, -230, 0, 24], [-144, 0, 84, 0, -113, 0, -12, 0]]
